/**
 * @license
 * Copyright (c) 2024 Handsoncode. All rights reserved.
 */
import { SearchStrategy } from '../../Lookup/SearchStrategy';
import { ProcedureAst } from '../../parser';
import { InterpreterState } from '../InterpreterState';
import { InterpreterValue, RawNoErrorScalarValue } from '../InterpreterValue';
import { SimpleRangeValue } from '../../SimpleRangeValue';
import { FunctionPlugin, FunctionPluginTypecheck, ImplementedFunctions } from './FunctionPlugin';
export declare class LookupPlugin extends FunctionPlugin implements FunctionPluginTypecheck<LookupPlugin> {
    static implementedFunctions: ImplementedFunctions;
    private rowSearch;
    /**
     * Corresponds to VLOOKUP(key, range, index, [sorted])
     *
     * @param ast
     * @param state
     */
    vlookup(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    /**
     * Corresponds to HLOOKUP(key, range, index, [sorted])
     *
     * @param ast
     * @param state
     */
    hlookup(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    match(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    protected searchInRange(key: RawNoErrorScalarValue, range: SimpleRangeValue, sorted: boolean, searchStrategy: SearchStrategy): number;
    private doVlookup;
    private doHlookup;
    private doMatch;
}
