/**
 * @license
 * Copyright (c) 2024 Handsoncode. All rights reserved.
 */
import { ProcedureAst } from '../../parser';
import { InterpreterState } from '../InterpreterState';
import { InterpreterValue } from '../InterpreterValue';
import { FunctionPlugin, FunctionPluginTypecheck, ImplementedFunctions } from './FunctionPlugin';
export declare class StatisticalPlugin extends FunctionPlugin implements FunctionPluginTypecheck<StatisticalPlugin> {
    static implementedFunctions: ImplementedFunctions;
    static aliases: {
        NEGBINOMDIST: string;
        EXPONDIST: string;
        BETADIST: string;
        NORMDIST: string;
        NORMINV: string;
        NORMSDIST: string;
        NORMSINV: string;
        LOGNORMDIST: string;
        LOGINV: string;
        TINV: string;
        HYPGEOMDIST: string;
        POISSON: string;
        WEIBULL: string;
        FINV: string;
        FDIST: string;
        CHIDIST: string;
        CHIINV: string;
        GAMMADIST: string;
        'GAMMALN.PRECISE': string;
        GAMMAINV: string;
        BETAINV: string;
        BINOMDIST: string;
        CONFIDENCE: string;
        CRITBINOM: string;
        WEIBULLDIST: string;
        TINV2T: string;
        TDISTRT: string;
        TDIST2T: string;
        FINVRT: string;
        FDISTRT: string;
        CHIDISTRT: string;
        CHIINVRT: string;
        LOGNORMINV: string;
        POISSONDIST: string;
    };
    erf(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    erfc(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    expondist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    fisher(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    fisherinv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    gamma(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    gammadist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    gammaln(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    gammainv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    gauss(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    betadist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    betainv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    binomialdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    binomialinv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    besselifn(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    besseljfn(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    besselkfn(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    besselyfn(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    chisqdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    chisqdistrt(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    chisqinv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    chisqinvrt(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    fdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    fdistrt(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    finv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    finvrt(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    weibulldist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    poissondist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    hypgeomdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tdist2t(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tdistrt(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tdistold(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tinv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    tinv2t(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    lognormdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    lognorminv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    normdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    norminv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    normsdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    normsinv(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    phi(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    negbinomdist(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    confidencenorm(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    confidencet(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
    standardize(ast: ProcedureAst, state: InterpreterState): InterpreterValue;
}
